@@ -17,4 +17,8 @@
 # USA
 #
 
-.onLoad <- function(lib, pkg) require(methods)
+.onLoad <- function(lib, pkg) {
+  require(methods)
+  pkgpath <- file.path(lib, pkg)
+  blendOptions(defaultSleighOptions, computeDefaultSleighOptions(pkgpath))
+}
